#
# Gramps - a GTK+/GNOME based genealogy program - Family Sheet plugin
#
# Copyright (C) 2008 Reinhard Mueller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# $Id: $

"""Reports/Text Reports/Family Sheet"""

#------------------------------------------------------------------------
#
# Standard Python modules
#
#------------------------------------------------------------------------
import string

#------------------------------------------------------------------------
#
# GRAMPS modules
#
#------------------------------------------------------------------------
from PluginUtils import register_report, PersonOption, BooleanOption, \
    EnumeratedListOption, relationship_class
from ReportBase import Report, ReportUtils, MenuReportOptions, \
    CATEGORY_TEXT, MODE_GUI, MODE_BKI, MODE_CLI
import BaseDoc
import DateHandler
# from TransUtils import sgettext as _
from BasicUtils import name_displayer
from gen.lib import EventType, Name


#------------------------------------------------------------------------
#
# FamilySheet report
#
#------------------------------------------------------------------------
class FamilySheet(Report):
    """
    A Family Sheet is a page which contains all available info about a specific
    person, the families this person is a father or mother in, and the children
    of these families.

    The intended use for this report is to get a full dump of the database in
    nice paper form in a way suitable to file it in a folder.

    Each Family Sheet contains a key at the top right which is derived from the
    relationship from the central person in the database and the person for
    which the sheet is printed. For direct ascendants, the Family Sheet key is
    the "Ahnentafel" number (also known as Eytzinger, Sosa, Sosa-Stradonitz, or
    Kekule number). Each child is assigned a letter starting from "a", and the
    Family Sheet key of the child is the Family Sheet key of the parent with
    the child's letter appended.

    The report contains full information (including all events, attributes,
    source references, and notes) for the key person and all its spouses.
    For children that had spouses, only a short section (including only name
    and birth event info) is printed along with a reference to the Family Sheet
    page on which this child would be the key person, while for children that
    had no spouses, full info is printed.

    If recursive printing is selected, each Family Sheet is followed by the
    Family Sheets of the children that had spouses.
    """

    def __init__(self, database, options_class):
        """
        Initialize the report.

        @param database: the GRAMPS database instance
        @param options_class: instance of the Options class for this report
        """

        Report.__init__(self, database, options_class)
        menu = options_class.menu
        self.person_id     = menu.get_option_by_name('pid').get_value()
        self.recurse       = menu.get_option_by_name('recurse').get_value()
        self.missing_info  = menu.get_option_by_name('missinginfo').get_value()
        self.underlinecall = menu.get_option_by_name('underlinecall').get_value()


    def write_report(self):
        """
        Build the actual report.
        """

        person = self.database.get_person_from_gramps_id(self.person_id)
        (rank, ahnentafel, person_key) = self.__calc_person_key(person)
        self.__process_person(person, rank, ahnentafel, person_key)


    def __process_person(self, person, rank, ahnentafel, person_key):
        """
        Recursively build the Family Sheet for this person and all children
        with spouses.

        @param person: Person object for the key person of the Family Sheet.
        @param rank: Numerical distance between the central person in the
            database and the person in the parameter (the number of births
            needed to connect them).
        @param ahnentafel: "Ahnentafel" number of the common ancestor of the
            central person in the database and the person in the parameter,
            seen from the side of the central person in the database.
        @param person_key: Family Sheet key to be printed on the top right of
            the corner.
        """

        # List of (person, rank, ahnentafel, person_key) tuples for persons to
        # process recursively after this one.
        more_sheets = []

        # Numbering of spouses (integer, but printed in roman numbers).
        spouse_index = 0

        # Numbering of children (integer, but printed as lowercase letters).
        child_index = 0

        # Source references to print as footnotes.
        self.__source_index = 0
        self.__sources = []

        # Notes to print as footnotes.
        self.__note_index = 0
        self.__notes = []

        # --- Now let the party begin! ---

        self.doc.start_paragraph('FSR-Key')
        self.doc.write_text(person_key)
        self.doc.end_paragraph()

        self.doc.start_table(None, 'FSR-Table')

        # Main person
        self.doc.start_row()
        self.doc.start_cell('FSR-HeadCell', 3)
        self.__dump_person(person, False, None)
        self.doc.end_cell()
        self.doc.end_row()

        # Spouses
        for family_handle in person.get_family_handle_list():
            family = self.database.get_family_from_handle(family_handle)

            spouse_index += 1

            spouse_handle = ReportUtils.find_spouse(person, family)
            spouse = self.database.get_person_from_handle(spouse_handle)

            # Determine relationship between the center person and the spouse.
            # If the spouse has a closer blood relationship than the current
            # person, we refer to the Family Sheet of the spouse instead of
            # printing the child list, because all children are more closely
            # related to the center person via the spouse than via the current
            # person. The same happens if the relationship is on the same
            # level, but the relationship via the spouse goes via a common
            # ancestor with a lower Ahnentafel numbering (i.e. a relationship
            # stronger father-sided). In these cases, refer_spouse will be set
            # to True.
            (spouse_rank, spouse_at, spouse_key) = \
                    self.__calc_person_key(spouse)
            if self.recurse != FamilySheetOptions.RECURSE_ALL:
                refer_spouse = (spouse_rank != -1 and \
                        (spouse_rank < rank or
                            (spouse_rank == rank and spouse_at < ahnentafel)))
            else:
                refer_spouse = False

            self.doc.start_row()

            self.doc.start_cell('FSR-NumberCell', 1)
            self.doc.start_paragraph('FSR-Number')
            self.doc.write_text(ReportUtils.roman(spouse_index))
            self.doc.end_paragraph()
            self.doc.end_cell()

            self.doc.start_cell('FSR-DataCell', 2)
            self.__dump_family(family, spouse)
            if refer_spouse:
                self.doc.start_paragraph('FSR-Normal')
                self.doc.write_text(_(u"\u2192 %s") % spouse_key)
                self.doc.end_paragraph()
            self.doc.end_cell()

            self.doc.end_row()

            if refer_spouse:
                # Spouse with closer relationship than current person? Don't
                # print children on this Family Sheet (but count them for the
                # numbering).
                child_index += len(family.get_child_ref_list())
                continue

            # Children
            for child_ref in family.get_child_ref_list():
                child = self.database.get_person_from_handle(child_ref.ref)
                child_letter = string.lowercase[child_index]

                self.doc.start_row()

                self.doc.start_cell('FSR-EmptyCell', 1)
                self.doc.end_cell()

                self.doc.start_cell('FSR-NumberCell', 1)
                self.doc.start_paragraph('FSR-Number')
                self.doc.write_text(child_letter)
                self.doc.end_paragraph()
                self.doc.end_cell()

                self.doc.start_cell('FSR-DataCell', 1)

                has_spouses = (child.get_family_handle_list() != [])

                self.__dump_person(child, has_spouses, child_ref)

                if has_spouses:
                    # We have to recalculate the key for this person, it could
                    # be closer related if it is a direct ancestor of the
                    # central person or one of its spouses.
                    (child_rank, child_at, child_key) = \
                            self.__calc_person_key(child)

                    self.doc.start_paragraph('FSR-Normal')
                    self.doc.write_text(_(u"\u2192 %s") % child_key)
                    self.doc.end_paragraph()

                    # We recursively print this child *only* if its
                    # relationship with the central person is closest via the
                    # current person. This way, we avoid that a person is
                    # printed recursively from more than one of its ancestors.
                    if child_key == person_key + child_letter or \
                            self.recurse == FamilySheetOptions.RECURSE_ALL:
                        more_sheets.append(
                                (child, child_rank, child_at, child_key))

                self.doc.end_cell()

                self.doc.end_row()

                child_index += 1

        self.doc.start_row()
        self.doc.start_cell('FSR-FootCell', 3)
        self.doc.end_cell()
        self.doc.end_row()

        self.doc.end_table()

        self.__dump_sources()
        self.__dump_notes()

        if self.recurse != FamilySheetOptions.RECURSE_NONE:
            for (child, child_rank, child_at, child_key) in more_sheets:
                self.doc.page_break()
                self.__process_person(child, child_rank, child_at, child_key)


    def __dump_family(self, family, spouse):
        """
        Output all data of a family the key person is a parent in, and all data
        of the corresponding spouse.
        """

        for attr in family.get_attribute_list():
            self.__dump_line(str(attr.get_type()), attr.get_value(), attr)

        for event_ref in family.get_event_ref_list():
            self.__dump_event(event_ref)

        if spouse:
            self.__dump_person(spouse, False, family)


    def __dump_person(self, person, short, ref):
        """
        Output all data of a person.

        @param person: Person object to output.
        @param short: If True, print only name and birth event.
        @param ref: Reference through which this person is linked into the
            Family Sheet. Can be a family object (for the spouses) or a
            child_ref object (for the children). Source references and notes
            for this reference object will also be output.
        """

        name = person.get_primary_name()
        name_text = self.__display_name(name)

        self.doc.start_paragraph('FSR-Name')
        mark = ReportUtils.get_person_mark(self.database, person)
        self.doc.write_text(name_text, mark)
        self.__write_sources(name)
        self.__write_notes(name)
        self.__write_sources(person)
        self.__write_notes(person)
        if ref:
            self.__write_sources(ref)
            self.__write_notes(ref)
        self.doc.end_paragraph()

        if short:
            event_ref = person.get_birth_ref()
            if event_ref:
                self.__dump_event(event_ref)
        else:
            for alt_name in person.get_alternate_names():
                name_type = str(alt_name.get_type())
                name = self.__display_name(alt_name)
                self.__dump_line(name_type, name, alt_name)

            for attr in person.get_attribute_list():
                self.__dump_line(str(attr.get_type()), attr.get_value(), attr)

            for event_ref in person.get_primary_event_ref_list():
                self.__dump_event(event_ref)

            for addr in person.get_address_list():
                location = ReportUtils.get_address_str(addr)
                date = DateHandler.get_date(addr)

                self.doc.start_paragraph('FSR-Normal')
                if date:
                    self.doc.write_text(_("Address (%(date)s): %(location)s") % {
                        'date': date,
                        'location': location})
                else:
                    self.doc.write_text(_("Address: %(location)s") % {
                        'location': location})
                self.__write_sources(addr)
                self.__write_notes(addr)
                self.doc.end_paragraph()


    def __display_name(self, name):
        """
        Build a string to output from a name object. This is really close to
        what BasicUtils.name_displayer.display_name does, but I wanted some
        hackish things not currently possible with that: underlining the call
        name if it is a part of the first_name, and replacing unknown name
        parts with underlines if placeholders for missing info are requested.
        """

        # Make a copy of the name object so we don't mess around with the real
        # data.
        n = Name(source=name)

        if self.missing_info:
            if not n.first_name:
                n.first_name = "____________"
            if not n.surname:
                n.surname = "____________"

        if n.call and self.underlinecall:
            if n.call in n.first_name:
                (before, after) = n.first_name.split(n.call)
                n.first_name = "%(before)s<u>%(call)s</u>%(after)s" % {
                        'before': before,
                        'call': n.call,
                        'after': after}
            else:
                n.first_name = "\"%(call)s\" (%(first)s)" % {
                        'call':  n.call,
                        'first': n.first_name}

        return name_displayer.display_name(n)


    def __dump_event(self, event_ref):
        """
        Output all data for an event.
        """

        event = self.database.get_event_from_handle(event_ref.ref)

        description = event.get_description()

        # Output placeholders for missing dates and places only for the
        # following event types.
        needs_date_place = event.get_type() in [
                EventType.BIRTH,
                EventType.DEATH,
                EventType.MARRIAGE,
                EventType.DIVORCE]

        date = event.get_date_object()
        date_text = DateHandler.displayer.display(date)
        if self.missing_info and needs_date_place:
            if date.is_empty():
                date_text = _("on %(placeholder)s") % {
                        'placeholder': "__________"}
            elif not date.is_regular():
                date_text = _("on %(placeholder)s (%(partial)s)") % {
                        'placeholder': "__________",
                        'partial': date_text}

        place_handle = event.get_place_handle()
        if place_handle:
            place_name = ReportUtils.place_name(self.database, place_handle)
        elif self.missing_info and needs_date_place:
            place_name = "____________"
        else:
            place_name = ""

        self.doc.start_paragraph('FSR-Normal')
        self.doc.write_text("%s:" % event.get_type())
        if description:
            self.doc.write_text(" ")
            self.doc.write_text(description)
        if date_text:
            self.doc.write_text(" ")
            self.doc.write_text(date_text)
        if place_name:
            self.doc.write_text(" ")
            self.doc.write_text(_("in %s") % place_name)
            if place_handle:
                place = self.database.get_place_from_handle(place_handle)
                self.__write_sources(place)
                self.__write_notes(place)
        for attr in event.get_attribute_list():
            self.doc.write_text(_("; %(type)s: %(value)s") % {
                'type'     : attr.get_type(),
                'value'    : attr.get_value()})
            self.__write_sources(attr)
            self.__write_notes(attr)
        self.__write_sources(event)
        self.__write_notes(event)
        self.__write_notes(event_ref)
        self.doc.end_paragraph()


    def __dump_line(self, name, text, object):
        """
        Output a name/text pair (like an attribute) with its related source
        references and notes.
        """

        self.doc.start_paragraph('FSR-Normal')
        self.doc.write_text("%s: %s" % (name, text))
        self.__write_sources(object)
        self.__write_notes(object)
        self.doc.end_paragraph()


    def __write_sources(self, obj):
        """
        Output source reference numbers for the given object (numbers like [1]
        in superscript) and collect the source references to be printed at the
        end of the report.
        """

        for source_ref in obj.get_source_references():
            self.__sources.append(source_ref)
            self.__source_index += 1
            self.doc.start_superscript()
            self.doc.write_text(" [%s]" % self.__source_index)
            self.doc.end_superscript()


    def __write_notes(self, obj):
        """
        Output note reference numbers for the given object (numbers like (1) in
        superscript) and collect the note handles to be printed at the end of
        the report.
        """

        for note_handle in obj.get_note_list():
            # Note already in list? If yes, use same number again.
            if note_handle in self.__notes:
                index = self.__notes.index(note_handle) + 1
            else:
                self.__notes.append(note_handle)
                self.__note_index += 1
                index = self.__note_index
            self.doc.start_superscript()
            self.doc.write_text(" (%s)" % index)
            self.doc.end_superscript()


    def __dump_sources(self):
        """
        Print the collected sources.
        """

        if self.__sources:
            self.doc.start_paragraph('FSR-Footnote')
            self.doc.write_text("\n")
            self.doc.write_text(_("Source references:"))
            self.doc.end_paragraph()

        index = 0
        for source_ref in self.__sources:
            source = self.database.get_source_from_handle(source_ref.ref)
            index += 1
            self.doc.start_paragraph('FSR-Footnote')
            self.doc.write_text("[%s]: " % index)
            if source.get_abbreviation():
                self.doc.write_text(source.get_abbreviation())
            else:
                self.doc.write_text(source.get_title())
            if source_ref.get_page():
                self.doc.write_text(_(", page %s") % source_ref.get_page())
            self.__write_notes(source_ref)
            self.doc.end_paragraph()


    def __dump_notes(self):
        """
        Print the collected notes.
        """

        if self.__notes:
            self.doc.start_paragraph('FSR-Footnote')
            self.doc.write_text("\n")
            self.doc.write_text(_("Notes:"))
            self.doc.end_paragraph()

        index = 0
        for note_handle in self.__notes:
            note = self.database.get_note_from_handle(note_handle)
            index += 1
            self.doc.start_paragraph('FSR-Footnote')
            self.doc.write_text("(%s): " % index)
            self.doc.write_text(note.get(False))
            self.doc.end_paragraph()


    def __calc_person_key(self, person):
        """
        The person key is a unique identifier that is built from the
        relationship to the default person. It consists of the "Ahnentafel"
        number of the common ancestor of the person with the default person,
        and then a letter representing the child number for each generation
        from the common ancestor to the person.

        If more than one common ancestor exists, the common ancestor with the
        lowest "Ahnentafel" number has precedence.

        For example, the second child of the third child of the father of the
        mother of the central person gets the person key "6cb".
        """

        relationship = relationship_class()

        default_person = self.database.get_default_person()

        # No home person set
        if default_person is None:
            return (-1, 0, "")

        # First try direct relationship.
        spousestring = ""
        info, msg = relationship.get_relationship_distance_new(
                self.database, default_person, person, all_dist=True)
        info = relationship.collapse_relations(info)[0]
        (rank, ancestor_handle, default_rel, default_fam, person_rel,
                person_fam) = info

        # Then try relationship to any spouse.
        if rank == -1:
            index = 0
            for family_handle in default_person.get_family_handle_list():
                index += 1
                family = self.database.get_family_from_handle(family_handle)
                spouse_handle = ReportUtils.find_spouse(default_person, family)
                spouse = self.database.get_person_from_handle(spouse_handle)
                info, msg = relationship.get_relationship_distance_new(
                        self.database, spouse, person, all_dist=True)
                info = relationship.collapse_relations(info)[0]
                (rank, ancestor_handle, default_rel, default_fam, person_rel,
                        person_fam) = info
                if rank != -1:
                    spousestring = ReportUtils.roman(index)
                    break
            # If no relationship found at all, exit here.
            if rank == -1:
                return (rank, 0, "")

        # Calculate Ahnentafel number of common ancestor.
        ahnentafel = 1
        for rel in default_rel:
            ahnentafel *= 2
            if rel in (relationship.REL_MOTHER,
                    relationship.REL_MOTHER_NOTBIRTH):
                ahnentafel += 1

        # Find out child letters.
        child = person
        childletters = ""
        for rel in person_rel:
            family_handle = child.get_main_parents_family_handle()
            family = self.database.get_family_from_handle(family_handle)
            if rel in (relationship.REL_MOTHER,
                    relationship.REL_MOTHER_NOTBIRTH):
                parent_handle = family.get_mother_handle()
            else:
                parent_handle = family.get_father_handle()
            parent = self.database.get_person_from_handle(parent_handle)
            # Count *all* children from this parent
            childletter = "?"
            index = 0
            for family_handle in parent.get_family_handle_list():
                family = self.database.get_family_from_handle(family_handle)
                for child_ref in family.get_child_ref_list():
                    if child_ref.ref == child.get_handle():
                        childletter = string.lowercase[index]
                        break
                    index += 1
                else:
                    continue
                break
            childletters = childletter + childletters
            child = parent

        return (rank, ahnentafel,
                "%s%s%s" % (spousestring, ahnentafel, childletters))


#------------------------------------------------------------------------
#
# MenuReportOptions
#
#------------------------------------------------------------------------
class FamilySheetOptions(MenuReportOptions):

    """
    Defines options and provides handling interface.
    """

    RECURSE_NONE = 0
    RECURSE_SIDE = 1
    RECURSE_ALL = 2

    def __init__(self, name, dbase):
        MenuReportOptions.__init__(self, name, dbase)


    def add_menu_options(self, menu):

        ##########################
        category_name = _("Report Options")
        ##########################

        pid = PersonOption(_("Center person"))
        pid.set_help(_("The person whose partners and children are printed"))
        menu.add_option(category_name, "pid", pid)

        recurse = EnumeratedListOption(_("Print sheets for:"),
                self.RECURSE_NONE)
        recurse.set_items([
            (self.RECURSE_NONE, _("Center person only")),
            (self.RECURSE_SIDE, _("Center person and descendants in side branches")),
            (self.RECURSE_ALL,  _("Center person and all descendants"))])
        menu.add_option(category_name, "recurse", recurse)

        missinginfo = BooleanOption(
                _("Print fields for missing information"), True)
        missinginfo.set_help(_("Whether to include fields for missing "
                               "information."))
        menu.add_option(category_name, "missinginfo", missinginfo)

        underlinecall = BooleanOption(
                _("Underline call name (only works with PDF output!)"), False)
        underlinecall.set_help(
                _("Whether to underline the call name if it is a part of the"
                  "first name."))
        menu.add_option(category_name, "underlinecall", underlinecall)


    def make_default_style(self, default_style):
        """Make default output style for the Family Sheet Report."""

        #Paragraph Styles
        font = BaseDoc.FontStyle()
        font.set_type_face(BaseDoc.FONT_SANS_SERIF)
        font.set_size(10)
        font.set_bold(0)
        para = BaseDoc.ParagraphStyle()
        para.set_font(font)
        para.set_description(_('The basic style used for the text display.'))
        default_style.add_paragraph_style('FSR-Normal', para)

        font = BaseDoc.FontStyle()
        font.set_type_face(BaseDoc.FONT_SANS_SERIF)
        font.set_size(10)
        font.set_bold(0)
        para = BaseDoc.ParagraphStyle()
        para.set_font(font)
        para.set_alignment(BaseDoc.PARA_ALIGN_RIGHT)
        para.set_description(_('The basic style used for the text display.'))
        default_style.add_paragraph_style('FSR-Key', para)

        font = BaseDoc.FontStyle()
        font.set_type_face(BaseDoc.FONT_SANS_SERIF)
        font.set_size(12)
        font.set_bold(1)
        para = BaseDoc.ParagraphStyle()
        para.set_font(font)
        para.set_description(_("The style used for names"))
        default_style.add_paragraph_style('FSR-Name', para)

        font = BaseDoc.FontStyle()
        font.set_type_face(BaseDoc.FONT_SANS_SERIF)
        font.set_size(12)
        font.set_bold(1)
        para = BaseDoc.ParagraphStyle()
        para.set_font(font)
        para.set_alignment(BaseDoc.PARA_ALIGN_CENTER)
        para.set_description(_("The style used for numbers"))
        default_style.add_paragraph_style('FSR-Number', para)

        font = BaseDoc.FontStyle()
        font.set_type_face(BaseDoc.FONT_SANS_SERIF)
        font.set_size(8)
        font.set_bold(0)
        para = BaseDoc.ParagraphStyle()
        para.set_font(font)
        para.set_description(_(
            'The style used for footnotes (notes and source references)'))
        default_style.add_paragraph_style('FSR-Footnote', para)

        #Table Styles
        cell = BaseDoc.TableCellStyle()
        cell.set_padding(0.1)
        cell.set_top_border(1)
        cell.set_left_border(1)
        cell.set_right_border(1)
        default_style.add_cell_style('FSR-HeadCell', cell)

        cell = BaseDoc.TableCellStyle()
        cell.set_padding(0.1)
        cell.set_left_border(1)
        default_style.add_cell_style('FSR-EmptyCell', cell)

        cell = BaseDoc.TableCellStyle()
        cell.set_padding(0.1)
        cell.set_top_border(1)
        cell.set_left_border(1)
        default_style.add_cell_style('FSR-NumberCell', cell)

        cell = BaseDoc.TableCellStyle()
        cell.set_padding(0.1)
        cell.set_top_border(1)
        cell.set_right_border(1)
        cell.set_left_border(1)
        default_style.add_cell_style('FSR-DataCell', cell)

        cell = BaseDoc.TableCellStyle()
        cell.set_padding(0.1)
        cell.set_top_border(1)
        default_style.add_cell_style('FSR-FootCell', cell)

        table = BaseDoc.TableStyle()
        table.set_width(100)
        table.set_columns(3)
        table.set_column_width(0, 7)
        table.set_column_width(1, 7)
        table.set_column_width(2, 86)
        default_style.add_table_style('FSR-Table', table)


#------------------------------------------------------------------------
#
# Translation hack
#
#------------------------------------------------------------------------
mytranslation = {
        u"\u2192 %s"                       : u"\u2192 %s",
        u"Address (%(date)s): %(location)s": u"Adresse (%(date)s): %(location)s",
        u"Address: %(location)s"           : u"Adresse: %(location)s",
        u"on %(placeholder)s"              : u"am %(placeholder)s",
        u"on %(placeholder)s (%(partial)s)": u"am %(placeholder)s (%(partial)s)",
        u"in %s"                           : u"in %s",
        u"Source references:"              : u"Quellenangaben:",
        u", page %s"                       : u", Seite %s",
        u"Notes:"                          : u"Notizen:"}

from gettext import gettext
import locale
lang = locale.getdefaultlocale()[0]
if lang:
    lang = lang.split('_')[0]
def _(string):
    if lang == 'de':
        return mytranslation.get(string, gettext(string))
    else:
        return gettext(string)


#------------------------------------------------------------------------
#
# Register the report
#
#------------------------------------------------------------------------
register_report(
    name = 'family_sheet',
    category = CATEGORY_TEXT,
    report_class = FamilySheet,
    options_class = FamilySheetOptions,
    modes = MODE_GUI | MODE_BKI | MODE_CLI,
    translated_name = _("Family Sheet"),
    status = _("Stable"),
    author_name = "Reinhard Mueller",
    author_email = "reinhard.mueller@bytewise.at",
    description = _("Produces a family sheet showing full information "
                    "about a person and his/her partners and children."))
